; @author Sylwester Arabas <slayoo@igf.fuw.edu.pl>

function IGFLibFastFSSPDataFile::readDataFromFile, columns_to_read, constraint_col = ccol, constraint_val = cval, limit = row_limit, constraint_linear = linear
  compile_opt idl2, strictarr

  ; opening the file
  get_lun, unit
  openr, unit, self.filename
  print, "reading data from file: ", self.filename, "..."

  line = ''
  count_in = 0L
  count_out = 0L
  count_comments = 0L
  num_rec = 0L
  allocated = 0
  tmpdata = dblarr(45, /nozero)
  data = ptrarr(n_elements(columns_to_read), /nozero)

  if keyword_set(linear) then begin 
    skip = 0L;
    lvalues = dblarr(2);
  endif

  while eof(unit) eq 0 and (not keyword_set(row_limit) or count_out lt row_limit) do begin
    readf, unit, format='(%"%s")', line
    count_in = count_in + 1
    ; data lines
    if (strmid(line,0,1) ne '#') then begin
      if keyword_set(linear) and (skip gt 0L) then begin
        skip = skip - 1L;
        continue;
      endif
      reads, line, tmpdata
      if keyword_set(linear) then begin 
        lvalues[0] = lvalues[1];
        lvalues[1] = tmpdata[ccol];
        if count_in > 2 then skip = (cval[count_out] - lvalues[1]) / (lvalues[1] - lvalues[0]) - 1L
      endif
      ; copy values to output array and handle costraint mechanism
      if not keyword_set(cval) or tmpdata[ccol] eq cval[count_out] then begin
        ; loop over columns to read 
        for i=0, n_elements(columns_to_read)-1 do begin
          ; assigning data to self.data
          (*(data[i]))[count_out] = tmpdata[columns_to_read[i]] 
        endfor
        ; incrementing data row counter
        count_out = count_out + 1
      endif
    ; comment lines
    endif else begin 
      count_comments = count_comments + 1
      if not allocated and strmid(line, 2, 9) eq 'startime=' then begin
        ; allocating memory for data storage - array dimensio based on heder information
        reads, strmid(line, 30, 10), format='(I)', num_rec;
        if (keyword_set("limit")) then num_rec = row_limit $
        else num_rec = num_rec * self.timeres / 1000; 
        for i=0, n_elements(columns_to_read)-1 do begin
          data[i] = ptr_new(dblarr(num_rec, /nozero));
        endfor
        allocated = 1;
      endif
    endelse
  endwhile
 
  print, count_in, " in";
  print, count_out, " out";
  print, count_comments, " comment lines";
  print, long(n_elements(columns_to_read)), " columns loaded";

  ; closing the file
  free_lun, unit;
  
  ; returning pointer to data
  return, ptr_new(data);
end
