; @author Sylwester Arabas <slayoo@igf.fuw.edu.pl>

pro fssp_compare
  ; compilation options
  compile_opt idl2, strictarr;

  ; welcome box
  spawn, "Xdialog --no-cancel --ok-label ""Run RICO FSSP Compare"" --icon rico_logo.xpm --msgbox """" 0 0";

  ; some X config
  red=[250,6,8,14,11,0,115,255,255,232,255,0]
  green=[250,0,100,178,175,255,237,244,206,3,0,0]
  blue=[250,255,255,255,17,0,30,1,14,207,0,0]

  device,decomp=0

  ; 
  window, 0, xsize=600, ysize=700
  TVLCT,red,green,blue

  ;
  window, 1, xsize=600, ysize=400
  TVLCT,red,green,blue

  ; 
  window, 2, xsize=600, ysize=200
  TVLCT,red,green,blue

  ; camera window
  window, 3, xsize=1056, ysize=480
  img704x480 = bytarr(3, 704, 480);
  img352x240 = bytarr(3, 352, 240);
  img176x120 = bytarr(3, 176, 120);

  !P.Color = 11
  color1 = 5 ; 1
  color2 = 3 ; 3

  datadir = "/home/pace/resu/RICO/";

  while (1) do begin

    ; display flight choice
    spawn, "Xdialog --stdout --cancel-label ""Exit"" --menubox ""Please select the flight..."" 0 0 10 rf06 16-12-2004 rf07 17-12-2004 rf08 19-12-2004 rf09 20-12-2004", flight;

    ; handle exit button
    flight = flight[0];
    if (flight eq "") then break;
  
    ffssp = obj_new('IGFLibFastFSSPData');
    file = file_search(datadir + flight + "/" + strupcase(flight) + "_hc*10");
    ffssp->setDataSourceMETEOFranceTextFile, file[0];

    fssp100 = obj_new('IGFLibFSSP100Data');   
    file = file_search(datadir + flight + "/" + "*HRT*.nc");
    fssp100->setDataSourceNCARNimbusNetCDF, file[0];

    ; constructing date string
    date = strmid(file_basename(file[0]), 33, 6);

    ; TODO: it works only for flights during one UTC day :( 
    bt = fssp100->getBaseTime();
    dt = bt mod 86400L;
    hh = fix(dt / 3600L);
    mm = fix((dt-hh*3600L) / 60L);
    ss = dt - hh * 3600L - mm * 60L;

    ; TODO: check if the given time is withing the flight-time bounds

    while (1) do begin

      ; ask for time
      spawn, "Xdialog --stdout "+$
        "--ok-label ""Focus"" "+$
        "--cancel-label ""Switch to other flight / Go back"" "+$
        "--timebox ""Flight "+flight+" selected, \nplease select a UTC time to focus on..."" "+$
        " 0 0 "+$
        string(hh)+" "+string(mm)+" "+string(ss)+" "+$
        " ", time;

      ; handle "switch to other flight" button
      time = time[0];
      if (time eq "") then break;

      ; handle time response from Xdialog
      reads, strmid(time,0,2), hh;
      reads, strmid(time,3,2), mm; 
      reads, strmid(time,6,2), ss;
      t = bt + hh * 3600L + mm * 60L + ss - dt 
      fssp100->focusOnTime, t
      ffssp->focusOnTime, hh * 3600L + mm * 60L + ss - ffssp->getBaseTime();

      ; color-coded histogram sets
      wset, 1
      erase
      fssp100->show, xpos=0, ypos=200, xsize=600, ysize=200
      ffssp->show, xpos=0, ypos=0, xsize=600, ysize=200

      ; controll-var plot
      wset, 2
      !P.MULTI = [0, 1, 1]
      fssp100->showControllVar, ytitle="LWC [g m-3]",  xtitle="time fine-tuning index"
      fssp100->showControllVar, /oplot, color=color1
      ffssp->showControllVar, /oplot, color=color2

      ; camera window
      wset, 3
      device, decomposed = 1;
      for ph = -7, 7 do begin
        img704x480[*] = 0;
        s = hh * 60L * 60 + mm * 60L + ss + ph
        file[0] = datadir + flight + "/camera/" + date + "_" + string(hh, format='(I02)') + "/c130_fwd" +$
          "_" + date + "_" + $
          strmid(string(100 + fix(s/60L/60L), format='(I3)'),1,2) + $
          strmid(string(100 + fix((s mod 3600L)/60L), format='(I3)'),1,2) + $
          strmid(string(100 + s mod 60, format='(I3)'),1,2) + $
          ".jpg";
        read_jpeg, file[0], img704x480;
        if (abs(ph) lt 2) then begin
          img352x240[0,*,*] = interpolate(img704x480[0,*,*], 2*indgen(352), 2*indgen(240), /grid);, cubic=-.5);
          img352x240[1,*,*] = interpolate(img704x480[1,*,*], 2*indgen(352), 2*indgen(240), /grid);, cubic=-.5);
          img352x240[2,*,*] = interpolate(img704x480[2,*,*], 2*indgen(352), 2*indgen(240), /grid);, cubic=-.5);
          tv, img352x240, 352*(1+ph), 120, true = 1;
        endif else begin
          img176x120[0,*,*] = interpolate(img704x480[0,*,*], 4*indgen(176), 4*indgen(120), /grid);, cubic=-.5);
          img176x120[1,*,*] = interpolate(img704x480[1,*,*], 4*indgen(176), 4*indgen(120), /grid);, cubic=-.5);
          img176x120[2,*,*] = interpolate(img704x480[2,*,*], 4*indgen(176), 4*indgen(120), /grid);, cubic=-.5);
          tv, img176x120, 176*(ph+(ph lt 0)*7+(ph gt 0)*(-2)), (ph lt 0) * 3*120, true = 1;
        endelse
      endfor
      device, decomposed = 0;
      xyouts, 10, 340, "tune = -10", /device
      xyouts, 10+352, 340, "tune = 0", /device
      xyouts, 10+2*352, 340, "tune = 10", /device

      ; initial tune values
      tune = ['0/0', 'unchecked']; 

      while (1) do begin

        split = strsplit(tune[0], "/", /extract);
        fssp100_tune = split[0];        
        ffssp_tune = split[1]; 

        ; two iterations (1 -> save, 2 -> plot)
        for iter=1, 2 do begin

          ; handling save option
          if (tune[1] ne "checked" and iter eq 1) then continue 
          
          if (iter eq 1) then begin 
            ; get filename from user
            spawn, "Xdialog --stdout "+$
              " --fselect """+flight+"_"+$
              string(hh, format='(I02)')+"_"+string(mm, format='(I02)')+"_"+string(ss, format='(I02)')+"_"+$
              string(fssp100_tune, format='(I03)')+"_"+string(ffssp_tune, format='(I03)')+".ps"""+$
              " 0 0 "+$
              " ", filename;

            ; handle cancel
            filename = filename[0];
            if (filename eq '') then continue

            ; set PostScript output parameters
            set_plot, 'ps' 
            device, /portrait, filename=filename
            !P.MULTI = [0, 1, 2]
            ; TODO: bug: it messes up the plotting layout (confirmed in GDL)

          endif else begin

            ;set_plot, 'x'
            wset, 0
            ;erase
            !P.MULTI = [0, 1, 3]

          endelse

          ; plot FSSP-100 data
          fssp100->show, finetune=fssp100_tune, xrange = [0, 50], charsize=2, ytitle="conc. density [cm-3 um-1]", subtitle="FSSP-100 @ "+time+"+"+string(fssp100_tune, format='(I03)')
          fssp100->show, finetune=fssp100_tune, color = color1, /oplot

          ; plot Fast-FSSP data
          ffssp->show, finetune=ffssp_tune, xrange = [0, 50], charsize=2, ytitle="conc. density [cm-3 um-1]", subtitle="Fast-FSSP @ "+time+"+"+string(ffssp_tune, format='(I03)')
          ffssp->show, finetune=ffssp_tune, color = color2, /oplot

          if (iter eq 2) then begin
        
            ; plot both spectra
            fssp100->show, finetune=fssp100_tune, xrange = [0, 50], yrange = [0, 50], charsize=2, ytitle="conc. density [cm-3 um-1]"
            fssp100->show, finetune=fssp100_tune, color = color1, /oplot
            ffssp->show, finetune=ffssp_tune, color = color2, /oplot
 
          endif else begin

            device, /close
   
          endelse

        endfor

        ; ask for subsecond timing
        spawn, "Xdialog --stdout "+$
          "--ok-label ""Show / Export"" "+$
          "--check ""Export spectra to PostScript file (BUG: messes up display no 0)"" "+$
          "--cancel-label ""Focus on other time / Go back"" "+$
          "--2rangesbox ""Flight "+flight+" selected, focused on "+time+", \nplease apply time fine-tuning"" "+$
          " 0 0 "+$
          " ""FSSP-100 time (10Hz resolution)"" -20 +19 "+string(fssp100_tune)+$
          " ""Fast-FSSP time (10Hz resolution)"" -20 +19 "+string(ffssp_tune)+$
          " ", tune

        ; handle "focus on other time" button 
        if (tune[0] eq "") then break;

      endwhile

    endwhile

  endwhile 

  print, "Bye..."

end
